package com.gic.auth.dto;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

/**
 * 
 * @ClassName: MenuDTO
 * @Description:  
 * @author zhiwj
 * @date 2020-09-09 20:34
 */
public class MenuDTO implements Serializable {
    private static final long serialVersionUID = -3884129728015209740L;

    /**
     * 新增页面参数校验分组
     */
    public interface SavePageValid {

    }

    /**
     * 新增gic子页面参数校验分组
     */
    public interface SaveGicChildPageValid {
    }

    /**
     * 编辑gic子页面参数校验分组
     */
    public interface UpdateGicChildPageValid {
    }

    /**
     * 编辑应用页面参数校验分组（应用页面是查询应用列表接口后自动新增）
     */
    public interface UpdateAppPageValid {
    }

    /**
     * 新增应用子页面参数校验分组
     */
    public interface SaveAppChildPageValid {
    }

    /**
     * 编辑应用子页面参数校验分组
     */
    public interface EditAppChildPageValid {
    }

    /**
     * 新增操作项参数校验分组
     */
    public interface SaveOperationItemValid {
    }

    /**
     * 编辑操作项参数校验分组
     */
    public interface UpdateOperationItemValid {
    }

    /**
     *
     */
    private Integer menuId;

    /**
     * 菜单名称
     */
    @NotBlank(message = "页面名称不能为空", groups = { SavePageValid.class, SaveGicChildPageValid.class,
            SaveAppChildPageValid.class, EditAppChildPageValid.class, SaveOperationItemValid.class,
            UpdateGicChildPageValid.class, UpdateOperationItemValid.class })
    private String menuName;

    /**
     * 所在项目
     */
    private String project;

    /**
     * 菜单路由
     */
    private String menuUrl;

    /**
     * 上级菜单id
     */
    @NotNull(message = "创建子页面，父级ID不能为空", groups = { SaveGicChildPageValid.class, SaveAppChildPageValid.class,
            SaveOperationItemValid.class })
    private Integer parentId;

    /**
     * 0当前页1新开页
     */
    @NotNull(message = "打开方式不能为空", groups = { SavePageValid.class, SaveGicChildPageValid.class,
            UpdateGicChildPageValid.class })
    private Integer target;

    /**
     *
     */
    private String iconUrl;

    /**
     *
     */
    private Integer sort;

    /**
     * 1删除0有效
     */
    private Integer deleteFlag;

    /**
     *
     */
    private Integer level;

    /**
     * 是否显示在菜单栏的菜单,0否1是
     */
    @NotNull(message = "页面类型（是否作为菜单）不能改为空", groups = { SavePageValid.class, SaveGicChildPageValid.class,
            UpdateGicChildPageValid.class })
    private Integer isShow;

    /**
     * 页面code
     */
    @NotBlank(message = "页面code不能为空", groups = { SavePageValid.class, SaveGicChildPageValid.class,
            UpdateAppPageValid.class, SaveAppChildPageValid.class, EditAppChildPageValid.class,
            SaveOperationItemValid.class, UpdateGicChildPageValid.class, UpdateOperationItemValid.class })
    private String menuCode;

    /**
     * 父级code
     */
    private String parentCode;

    /**
     * 多选，例如_1_2_格式，根据应用版本
     */
    @NotBlank(message = "版本不能为空", groups = { SavePageValid.class, SaveGicChildPageValid.class,
            SaveAppChildPageValid.class, SaveOperationItemValid.class, UpdateGicChildPageValid.class,
            EditAppChildPageValid.class, UpdateOperationItemValid.class })
    private String menuVersion;

    /**
     * 0:页面 1:权限项 2:操作项
     */
    private Integer menuType;

    /**
     * 应用版本名称
     */
    private String menuVersionName;

    private List<MenuDTO> children;
    /**
     * 是否需要同步生成权限项（新增操作项的时候的可选项）
     */
    private Boolean needCreateAuthItem = false;

    /**
     * 前端使用：二级域名，也就是项目的部署路径，比如门店项目部署在/damo-store下面，那么就是/damo-store
     */
    private String projectUrlForWeb;

    /**
     * 操作项 ，仅超管可用 1:是 0:否
     */
    private Integer adminOnlySign;

    /**
     * 1:管理员权限集 2:商户级超管及达摩侧人员 3:仅达摩侧人员
     */
    private Integer authType;

    /**
     * 部门职位
     */
    private List<MenuAuthDepartDTO> menuAuthDepartList;

    /**
     * 0:gic 1:好办后台 2：好办小程序
     */
    private Integer platformType;

    public Integer getMenuId() {
        return menuId;
    }

    public void setMenuId(Integer menuId) {
        this.menuId = menuId;
    }

    public String getMenuName() {
        return menuName;
    }

    public void setMenuName(String menuName) {
        this.menuName = menuName;
    }

    public String getProject() {
        return project;
    }

    public void setProject(String project) {
        this.project = project;
    }

    public String getMenuUrl() {
        return menuUrl;
    }

    public void setMenuUrl(String menuUrl) {
        this.menuUrl = menuUrl;
    }

    public Integer getParentId() {
        return parentId;
    }

    public void setParentId(Integer parentId) {
        this.parentId = parentId;
    }

    public Integer getTarget() {
        return target;
    }

    public void setTarget(Integer target) {
        this.target = target;
    }

    public String getIconUrl() {
        return iconUrl;
    }

    public void setIconUrl(String iconUrl) {
        this.iconUrl = iconUrl;
    }

    public Integer getSort() {
        return sort;
    }

    public void setSort(Integer sort) {
        this.sort = sort;
    }

    public Integer getDeleteFlag() {
        return deleteFlag;
    }

    public MenuDTO setDeleteFlag(Integer deleteFlag) {
        this.deleteFlag = deleteFlag;
        return this;
    }

    public Integer getLevel() {
        return level;
    }

    public void setLevel(Integer level) {
        this.level = level;
    }

    public Integer getIsShow() {
        return isShow;
    }

    public void setIsShow(Integer isShow) {
        this.isShow = isShow;
    }

    public List<MenuDTO> getChildren() {
        return children;
    }

    public void setChildren(List<MenuDTO> children) {
        this.children = children;
    }

    public String getMenuCode() {
        return menuCode;
    }

    public void setMenuCode(String menuCode) {
        this.menuCode = menuCode;
    }

    public String getParentCode() {
        return parentCode;
    }

    public void setParentCode(String parentCode) {
        this.parentCode = parentCode;
    }

    public String getMenuVersion() {
        return menuVersion;
    }

    public void setMenuVersion(String menuVersion) {
        this.menuVersion = menuVersion;
    }

    public Integer getMenuType() {
        return menuType;
    }

    public void setMenuType(Integer menuType) {
        this.menuType = menuType;
    }

    public String getMenuVersionName() {
        return menuVersionName;
    }

    public void setMenuVersionName(String menuVersionName) {
        this.menuVersionName = menuVersionName;
    }

    public Boolean isNeedCreateAuthItem() {
        return needCreateAuthItem;
    }

    public void setNeedCreateAuthItem(Boolean needCreateAuthItem) {
        this.needCreateAuthItem = needCreateAuthItem;
    }

    public String getProjectUrlForWeb() {
        return projectUrlForWeb;
    }

    public void setProjectUrlForWeb(String projectUrlForWeb) {
        this.projectUrlForWeb = projectUrlForWeb;
    }

    public Integer getAdminOnlySign() {
        return adminOnlySign;
    }

    public void setAdminOnlySign(Integer adminOnlySign) {
        this.adminOnlySign = adminOnlySign;
    }

    public List<MenuAuthDepartDTO> getMenuAuthDepartList() {
        return menuAuthDepartList;
    }

    public MenuDTO setMenuAuthDepartList(List<MenuAuthDepartDTO> menuAuthDepartList) {
        this.menuAuthDepartList = menuAuthDepartList;
        return this;
    }

    public Integer getAuthType() {
        return authType;
    }

    public MenuDTO setAuthType(Integer authType) {
        this.authType = authType;
        return this;
    }

    public Integer getPlatformType() {
        return platformType;
    }

    public MenuDTO setPlatformType(Integer platformType) {
        this.platformType = platformType;
        return this;
    }

    @Override
    public String toString() {
        return "MenuDTO{" +
                "menuId=" + menuId +
                ", menuName='" + menuName + '\'' +
                ", project='" + project + '\'' +
                ", menuUrl='" + menuUrl + '\'' +
                ", parentId=" + parentId +
                ", target=" + target +
                ", iconUrl='" + iconUrl + '\'' +
                ", sort=" + sort +
                ", status=" + deleteFlag +
                ", level=" + level +
                ", isShow=" + isShow +
                ", menuCode='" + menuCode + '\'' +
                ", parentCode='" + parentCode + '\'' +
                ", menuVersion='" + menuVersion + '\'' +
                ", menuType=" + menuType +
                ", menuVersionName='" + menuVersionName + '\'' +
                ", children=" + children +
                ", needCreateAuthItem=" + needCreateAuthItem +
                ", projectUrlForWeb='" + projectUrlForWeb + '\'' +
                ", adminOnlySign=" + adminOnlySign +
                ", authType=" + authType +
                ", menuAuthDepartList=" + menuAuthDepartList +
                ", platformType=" + platformType +
                '}';
    }
}
