package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.qo.AuditorListQO;

import java.util.List;

/**
 * 审核员
 * @ClassName: AuditorApiService
 * @Description:  
 * @author zhiwj
 * @date 2019-09-19 11:41
 */
public interface AuditorApiService {

    /**
     * 保存审核员
     * @Title: saveOrUpdateAuditor
     * @Description:
     * @author zhiwj
     * @param auditorDTO
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.String>
     * @throws
     */
    ServiceResponse<String> saveOrUpdateAuditor(AuditorDTO auditorDTO);

    /**
     * 删除审核员
     * @Title: delAuditor
     * @Description:
     * @author zhiwj
     * @param auditorId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<Void> delAuditor(Integer auditorId);

    /**
     * getAuditor
     * @Title: getAuditor
     * @Description:
     * @author zhiwj
     * @param auditorId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Integer>
     * @throws
     */
    ServiceResponse<AuditorDTO> getAuditor(Integer auditorId);

    /**
     * 获取审核员的服务号二维码
     * @param auditorId
     * @return
     */
    ServiceResponse<String> getQrcodeUrl(Integer auditorId);

    /**
     * pageAuditor
     * @Title: pageAuditor
     * @Description:
     * @author zhiwj
     * @param auditorListQO
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.AuditorDTO>>
     * @throws
     */
    ServiceResponse<Page<AuditorDTO>> pageAuditor(AuditorListQO auditorListQO);

    /**
     * 接收扫描二维码之后的回调
     * @Title: receiveOpenid
     * @Description:
     * @author zhiwj
     * @param qrCodeParam
     * @param openid
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> receiveOpenid(String qrCodeParam, String openid);

    /**
     * 不分頁列表
     * @Title: listAuditor
     * @Description:
     * @author zhiwj
     * @param auditorListQO
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AuditorDTO>>
     * @throws
     */
    ServiceResponse<List<AuditorDTO>> listAuditor(AuditorListQO auditorListQO);

    /**
     * 查所有的审核员
     * @Title: listAuditorByProject
     * @Description:
     * @author zhiwj
     * @param projectItemId 审批项id
     * @param userId 管理员id
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AuditorDTO>>
     * @throws
     */
    ServiceResponse<List<AuditorDTO>> listAuditorByProject(Integer projectItemId, Integer userId, Integer enterpriseId);

    /**
     * 查所有的审核员
     * @Title: listAuditorByProject
     * @Description:
     * @author zhiwj
     * @param projectItemCode
     * @param userId
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AuditorDTO>>
     * @throws
     */
    ServiceResponse<List<AuditorDTO>> listAuditorByProject(String projectItemCode, Integer userId, Integer enterpriseId);
    
    /**
     * listAuditorByProject
     * @Title: listAuditorByProject
     * @Description: 
     * @author zhiwj 
     * @param projectItemCode
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AuditorDTO>>
     * @throws 
     */
    ServiceResponse<List<AuditorDTO>> listAuditorByProject(String projectItemCode, Integer enterpriseId);

    /**
     * 通过openid查询审核员
     * @Title: getAuditorByOpenid
     * @Description:
     * @author zhiwj
     * @param openid
     * @param enterpriseId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.AuditorDTO>
     * @throws
     */
    ServiceResponse<AuditorDTO> getAuditorByOpenid(String openid, Integer enterpriseId);

    /**
     * delRelByProjectItemId
     * @Title: delRelByProjectItemId
     * @Description:
     * @author zhiwj
     * @param projectItemId
     * @return com.gic.api.base.commons.ServiceResponse<java.lang.Void>
     * @throws
     */
    ServiceResponse<Void> delRelByProjectItemId(Integer projectItemId);

    /**
     * 通过userId查询审核员
     * @Title: getAuditorByUserId
     * @Description:
     * @author zhiwj
     * @param enterpriseId
     * @param userId
     * @return com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.AuditorDTO>
     * @throws
     */
    ServiceResponse<AuditorDTO> getAuditorByUserId(Integer enterpriseId, Integer userId);

    /**
     * 管理员数据修改同步到审核员
     * @param userId
     * @param enterpriseId
     * @param phone
     * @param name
     * @param openId
     * @return
     */
    ServiceResponse<Void> syncModifyNameOrPhone(Integer userId, Integer enterpriseId, String phone, String name, String openId);

}
