package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AppDTO;
import com.gic.auth.dto.CollaboratorDTO;
import com.gic.auth.dto.CollaboratorListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.qo.CollaboratorListQO;

import java.util.List;

/**
 * 协作人接口
 * @ClassName: CollaboratorApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2019/11/1 4:15 PM 
 */
public interface CollaboratorApiService {

    /**
     * 新增协作人
     * @Title: save 
     * @Description:
      * @author guojuxing
     * @param dto 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
      */
    ServiceResponse<Integer> save(CollaboratorDTO dto);

    /**
     * 分页查询协作人泪奔
     * @Title: page 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.CollaboratorListDTO>> 
      */
    ServiceResponse<Page<CollaboratorListDTO>> page(CollaboratorListQO params);

    /**
     * 查询已购买的一级应用
     * @Title: listApp 
     * @Description:
      * @author guojuxing
     * @param enterpriseId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.AppDTO> 
      */
    ServiceResponse<List<AppDTO>> listApp(Integer enterpriseId);

    /**
     * 查询详情
     * @Title: getDetail 
     * @Description:
      * @author guojuxing
     * @param collaboratorId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.CollaboratorDTO> 
      */
    ServiceResponse<CollaboratorDTO> getDetail(Integer collaboratorId);

    /**
     * 授权
     * @Title: auth 
     * @Description:
      * @author guojuxing
     * @param subAppIds 子应用ID，英文逗号隔开
     * @param menuIds  操作权限ID，英文逗号隔开
     * @param resourcesIds 资源组ID，英文逗号隔开
     * @param collaboratorId
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.String>  返回日志操作对象
      */
    ServiceResponse<String> auth(String subAppIds, String menuIds, String resourcesIds, Integer collaboratorId);

    /**
     * 取消
     * @Title: cancel 
     * @Description:
      * @author guojuxing
     * @param collaboratorId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    ServiceResponse<Void> cancel(Integer collaboratorId);

    /**
     * 获取协作应用的应用菜单权限页面
     * @Title: getCollaboratorMenuPage 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param phone
     * @param appId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> getCollaboratorMenuPage(Integer enterpriseId, String phone, String appId);

    /**
     * 获取协作应用的没有权限的操作项列表
     * @Title: getCollaboratorOperationItem 
     * @Description:
     * @author guojuxing
     * @param enterpriseId
     * @param phone
     * @param appId 
     * @param userId 登录用户ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> getCollaboratorOperationItem(Integer enterpriseId, String phone, String appId, Integer userId);

    /**
     * 查询主键ID，协作人那边授权跳转，需要主键ID
     * @param enterpriseId 协作商户ID
     * @param phone
     * @param appId
     * @return
     */
    ServiceResponse<Integer> getIdByPhoneAndAppId(Integer enterpriseId, String phone, String appId);

    /**
     * 删除
     * @param enterpriseId 写作商户ID
     * @param phone
     * @param appId
     * @return
     */
    ServiceResponse<Void> deleteByPhoneAndAppId(Integer enterpriseId, String phone, String appId);
}
