package com.gic.auth.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.MenuItemDTO;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;

import java.util.List;
/**
 * 好办菜单
 * @ClassName: MenuForHaoBanApiService 
 * @Description:  
 * @author guojuxing 
 * @date 2020/7/30 2:06 PM 
 */
public interface MenuForHaoBanApiService {

    /**
     * 新增菜单
     * @Title: savePage 
     * @Description:
     * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> savePage(MenuDTO menuDTO);

    /**
     * 编辑菜单 （
     * @Title: updatePage 
     * @Description:
     * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> updatePage(MenuDTO menuDTO);

    /**
     * 删除菜单
     * @Title: deleteMenu 
     * @Description:
     * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> deleteMenu(Integer menuId);

    /**
     * 新增子菜单
     * @Title: saveChildPage 
     * @Description:
     * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveChildPage(MenuDTO menuDTO);

    /**
     * 编辑子菜单
     * @Title: updateChildPage 
     * @Description:
     * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> updateChildPage(MenuDTO menuDTO);

    /**
     * 新增操作项
     * @Title: saveOperationItem 
     * @Description:
     * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Integer> 
     */
    ServiceResponse<Integer> saveOperationItem(MenuDTO menuDTO);

    /**
     * 编辑操作项
     * @Title: updateOperationItem 
     * @Description:
     * @author guojuxing
     * @param menuDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO);

    /**
     * 分页查询操作项
     * @Title: pageMenuOperationItem 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.api.base.commons.Page<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params);

    /**
     * 详情
     * @Title: getDetailById 
     * @Description:
     * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.MenuDTO> 
     */
    ServiceResponse<MenuDTO> getDetailById(Integer menuId);

    /**
     * 新增权限项
     * @Title: saveAuthItem 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> saveAuthItem(AuthItemSaveQO params);

    /**
     * 编辑权限项
     * @Title: updateAuthItem 
     * @Description:
     * @author guojuxing
     * @param params 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
     */
    ServiceResponse<Void> updateAuthItem(AuthItemSaveQO params);

    /**
     * 查看菜单下的权限项
     * @Title: listAuthItemByMenuId 
     * @Description:
     * @author guojuxing
     * @param menuId 
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.AuthItemListDTO>> 
     */
    ServiceResponse<List<AuthItemListDTO>> listAuthItemByMenuId(Integer menuId);

    /**
     * 根据code查询，因为code唯一
     * @Title: getMenuByMenuCode 
     * @Description:
     * @author guojuxing
     * @param menuCode 
     * @return  com.gic.api.base.commons.ServiceResponse<com.gic.auth.dto.MenuDTO> 
     */
    ServiceResponse<MenuDTO> getMenuByMenuCode(String menuCode);

   /**
    * 查询菜单数数据
    * @Title: listMenu 
    * @Description:
    * @author guojuxing
    * @param search 页面名称/code
    * @param menuType 0:页面 1：权限项 2：操作项
    * @param platformType  0:gic 1:好办后台 2：好办小程序
    * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
    */
    ServiceResponse<List<MenuDTO>> listMenu(String search, Integer menuType, Integer platformType);

    /**
     * 查询页面下面的操作项列表数据
     * @Title: listMenuItemByParentId 
     * @Description:
     * @author guojuxing
     * @param parentId 菜单ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuDTO>> 
     */
    ServiceResponse<List<MenuDTO>> listMenuItemByParentId(Integer parentId);

    /**
     * 查询权限项下面的操作项数据
     * @Title: listMenuItemByMenuId 
     * @Description:
     * @author guojuxing
     * @param menuId 权限项ID
     * @return  com.gic.api.base.commons.ServiceResponse<java.util.List<com.gic.auth.dto.MenuItemDTO>> 
     */
    ServiceResponse<List<MenuItemDTO>> listMenuItemByMenuId(Integer menuId);
}
