package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuthItemListDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.MenuItemDTO;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.MenuForHaoBanApiService;
import com.gic.auth.service.MenuItemApiService;
import com.gic.enterprise.error.ErrorCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
/**
 * 
 * @ClassName: MenuForHaoBanApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:37
 */
@Service("menuForHaoBanApiService")
public class MenuForHaoBanApiServiceImpl implements MenuForHaoBanApiService{
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private MenuItemApiService menuItemApiService;

    @Override
    public ServiceResponse<Void> savePage(MenuDTO menuDTO) {
        return menuApiService.savePageForHaoban(menuDTO);
    }

    @Override
    public ServiceResponse<Void> updatePage(MenuDTO menuDTO) {
        return menuApiService.updateGICPage(menuDTO);
    }

    @Override
    public ServiceResponse<Void> deleteMenu(Integer menuId) {
        return menuApiService.deleteMenu(menuId);
    }

    @Override
    public ServiceResponse<Integer> saveChildPage(MenuDTO menuDTO) {
        return menuApiService.saveGICChildPage(menuDTO);
    }

    @Override
    public ServiceResponse<Void> updateChildPage(MenuDTO menuDTO) {
        return menuApiService.updateGICChildPage(menuDTO);
    }

    @Override
    public ServiceResponse<Integer> saveOperationItem(MenuDTO menuDTO) {
        return menuApiService.saveOperationItem(menuDTO);
    }

    @Override
    public ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO) {
        return menuApiService.updateOperationItem(menuDTO);
    }

    @Override
    public ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params) {
        if (!isRightPlatformType(params.getPlatformType())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "平台类型值非法");
        }
        return menuApiService.pageMenuOperationItem(params);
    }

    @Override
    public ServiceResponse<MenuDTO> getDetailById(Integer menuId) {
        return menuApiService.getDetailById(menuId);
    }

    @Override
    public ServiceResponse<Void> saveAuthItem(AuthItemSaveQO params) {
        if (!isRightPlatformType(params.getPlatformType())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "平台类型值非法");
        }
        return menuApiService.saveAuthItem(params);
    }

    @Override
    public ServiceResponse<Void> updateAuthItem(AuthItemSaveQO params) {
        return menuApiService.updateAuthItem(params);
    }

    @Override
    public ServiceResponse<List<AuthItemListDTO>> listAuthItemByMenuId(Integer menuId) {
        return menuApiService.listAuthItemByMenuId(menuId);
    }

    @Override
    public ServiceResponse<MenuDTO> getMenuByMenuCode(String menuCode) {
        return menuApiService.getMenuByMenuCode(menuCode);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenu(String search, Integer menuType, Integer platformType) {
        if (!isRightPlatformType(platformType)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "平台类型值非法");
        }
        MenuListQO params = new MenuListQO();
        params.setSearch(search);
        params.setMenuType(menuType);
        params.setPlatformType(platformType);
        return menuApiService.listMenuTreeForHaoban(params);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuItemByParentId(Integer parentId) {
        return menuApiService.listMenuItemByParentId(parentId);
    }

    @Override
    public ServiceResponse<List<MenuItemDTO>> listMenuItemByMenuId(Integer menuId) {
        return menuItemApiService.listMenuItemByMenuId(menuId);
    }

    private static boolean isRightPlatformType(Integer platformType) {
        return platformType != null && (platformType.intValue() == 1
                || platformType.intValue() == 2
                || platformType.intValue() == 3
                || platformType.intValue() == 4);
    }
}
