package com.gic.auth.service.outer.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.application.center.api.dto.ApplicationResourceDTO;
import com.gic.application.center.api.service.AuthorizeService;
import com.gic.auth.dto.CustomUserAreaDTO;
import com.gic.auth.dto.ResourceGroupCustomUserDTO;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.dto.open.*;
import com.gic.auth.service.OrderAppResourceService;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.ResourceOpenApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.gateway.sdk.anno.GatewayExporter;
import com.gic.gateway.sdk.anno.GatewayParams;
import com.gic.goods.api.dto.selector.GoodsRightsSelectorWithRightsDTO;
import com.gic.goods.api.service.GoodsRightsSelectorApiService;
import com.gic.member.config.api.dto.ChannelAreaInfoDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.AuthorizationEnterConfApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.member.filter.api.service.CuIndexOperaApiService;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreWidgetApiService;

/**
 * 
 * @ClassName: ResourceOpenApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:38
 */
@Service("resourceOpenApiService")
@GatewayExporter
public class ResourceOpenApiServiceImpl implements ResourceOpenApiService{
    private static final Logger LOGGER = LogManager.getLogger(ResourceOpenApiServiceImpl.class);

    @Autowired
    private AuthorizeService authorizeService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;
    @Autowired
    private CuIndexOperaApiService cuIndexOperaApiService;
    @Autowired
    private OrderAppResourceService orderAppResourceService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private AuthorizationEnterConfApiService authorizationEnterConfApiService;
    @Autowired
    private GoodsRightsSelectorApiService goodsRightsSelectorApiService;

    @Override
    @GatewayParams(query = {"appId", "enterpriseId"})
    public ServiceResponse<ResourceOpenDTO> getResource(String appId, Integer enterpriseId) {
        ResourceGroupDTO resourceDTO = getResourceDTO(appId, enterpriseId);
        ResourceOpenDTO result = new ResourceOpenDTO();
        //门店资源
        result.setStore(getStoreResource(enterpriseId, resourceDTO.getStoreResource()));
        //用户资源
        CustomUserAreaDTO customUserAreaDTO = EntityUtil.changeEntityNew(CustomUserAreaDTO.class, resourceDTO);
        List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(resourceDTO.getCustomUserData(),
                ResourceGroupCustomUserDTO.class);
        customUserAreaDTO.setCustomUserList(customUserList);
        result.setCustomUserAreaDTO(customUserAreaDTO);
        //商品资源
        result.setGoodsResource(getGoodsResource(enterpriseId, resourceDTO.getGoodsResourceId()));
        return ServiceResponse.success(result);
    }

    private GoodsResourceDTO getGoodsResource(Integer enterpriseId, Long goodsResource) {
        if (goodsResource != null) {
            GoodsResourceDTO dto = new GoodsResourceDTO();
            dto.setGoodsResource(goodsResource);
            if (goodsResource != null) {
                ServiceResponse<GoodsRightsSelectorWithRightsDTO> result = goodsRightsSelectorApiService
                        .getGoodsRightsSelectorWithRights(goodsResource, enterpriseId);
                LOGGER.info("对外商品资源查询接口:{}", JSON.toJSONString(result));
                if (result.isSuccess()) {
                    GoodsRightsSelectorWithRightsDTO goodsRightsSelectorWithRightsDTO = result.getResult();
                    GoodsRightsAuthDTO goodsRightsAuth = EntityUtil.changeEntityNew(GoodsRightsAuthDTO.class, goodsRightsSelectorWithRightsDTO);
                    dto.setGoodsRightsAuth(goodsRightsAuth);
                    return dto;
                }
            }
        }
        return null;
    }

    private UserResourceDTO getUserResource(Integer enterpriseId, Long userResourceId) {
        if (userResourceId != null) {
            if (userResourceId != null) {
                ServiceResponse<List<ChannelAreaInfoDTO>> userResult = authorizationEnterConfApiService.getAreaInfoByScreenWidget(userResourceId, enterpriseId);
                LOGGER.info("对外用户资源查询接口:{}", JSON.toJSONString(userResult));
                if (userResult.isSuccess()) {
                    UserResourceDTO dto = new UserResourceDTO();
                    dto.setUserResource(userResourceId);
                    List<ChannelAreaInfoDTO> userResource = userResult.getResult();
                    if (CollectionUtils.isNotEmpty(userResource)) {
                        List<UserAreaDTO> userAreaList = new ArrayList<>();
                        for (ChannelAreaInfoDTO channelAreaInfoDTO : userResource) {
                            userAreaList.add(new UserAreaDTO(channelAreaInfoDTO.getAreaId(), channelAreaInfoDTO.getName(), channelAreaInfoDTO.getCuType()));
                        }
                        dto.setUserArea(new UserAreaAuthDTO(userAreaList));
                        return dto;
                    }
                }
            }
        }
        return null;
    }


    private StoreResourceDTO getStoreResource(Integer enterpriseId, Long storeResourceId) {
        List<StoreOpenDTO> storeList = new ArrayList<>();
        if (storeResourceId != null) {
            //查询门店资源组所有的门店权限
            ServiceResponse<Page<StoreDTO>> storeResult = storeWidgetApiService
                    .listStoreByStoreWidgetId(enterpriseId, storeResourceId.intValue(), 1, 10000);
            if (storeResult.isSuccess()) {
                Page<StoreDTO> page = storeResult.getResult();
                if (page == null) {
                    return null;
                }
                List<StoreDTO> list = page.getResult();
                Map<String, String> regionMap = new HashMap<>(16);
                if (CollectionUtils.isNotEmpty(list)) {
                    for (StoreDTO storeDTO : list) {
                        regionMap.put(storeDTO.getRegionId().toString(), storeDTO.getRegionName());
                    }
                    for(Map.Entry<String, String> entry : regionMap.entrySet()) {
                        storeList.add(new StoreOpenDTO(Integer.parseInt(entry.getKey()), entry.getValue()));
                    }
                    StoreAuthDTO storeAuth = new StoreAuthDTO(storeList);
                    return new StoreResourceDTO(storeResourceId, storeAuth);
                }
            }
        }
        return null;
    }

    private ResourceGroupDTO getResourceDTO(String appId, Integer enterpriseId) {
        ServiceResponse<ApplicationResourceDTO> res = authorizeService
                .findResourceByEnterpriseIdAndAppId(Long.valueOf(enterpriseId), appId);
        LOGGER.info("对外门店资源查询接口:{}", JSON.toJSONString(res));
        if (res.isSuccess()) {
            ApplicationResourceDTO resourceDTO = res.getResult();
            if (resourceDTO != null) {
                Integer resourceId = resourceDTO.getResourceId();
                if (resourceId == null) {
                    return null;
                }
                ServiceResponse<ResourceGroupDTO> resourceResult = resourceGroupApiService.getResourceGroup(resourceId);
                LOGGER.info("对外API门店资源组数据:{}", JSON.toJSONString(resourceResult));
                if (resourceResult.isSuccess()) {
                    return resourceResult.getResult();
                }
            }
        }
        return null;
    }
}
