package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.AuthorizationStatusEnum;
import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.constant.ResourceTypeEnum;
import com.gic.auth.dto.UnionEnterpriseDTO;
import com.gic.auth.dto.UnionEnterpriseResourceDTO;
import com.gic.auth.entity.TabSysUnionEnterprise;
import com.gic.auth.entity.TabSysUnionEnterpriseResource;
import com.gic.auth.service.UnionEnterpriseApiService;
import com.gic.auth.service.UnionEnterpriseResourceService;
import com.gic.auth.service.UnionEnterpriseService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constants.Constants;
import com.gic.store.service.StoreAuthorizationApiService;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
/**
 * 
 * @ClassName: UnionEnterpriseApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:45
 */
@Service("unionEnterpriseApiService")
public class UnionEnterpriseApiServiceImpl implements UnionEnterpriseApiService {
    @Autowired
    private UnionEnterpriseService unionEnterpriseService;
    @Autowired
    private UnionEnterpriseResourceService unionEnterpriseResourceService;
    @Autowired
    private StoreAuthorizationApiService storeAuthorizationApiService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> saveUnionEnterprise(UnionEnterpriseDTO enterpriseDTO) {
        TabSysUnionEnterprise unionEnterprise = EntityUtil.changeEntityByJSON(TabSysUnionEnterprise.class, enterpriseDTO);
        int i = 0;
        if(enterpriseDTO.getUnionId() == null) {
            unionEnterprise.setCreateTime(new Date());
            unionEnterprise.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);
            unionEnterprise.setAuthorizationStatus(AuthorizationStatusEnum.NORMAL.getCode());
            unionEnterprise.setUpdateTime(new Date());
            i = this.unionEnterpriseService.saveUnionEnterprise(unionEnterprise);
        }else {
            i = this.unionEnterpriseService.updateUnionEnterprise(unionEnterprise);
        }
        if (i > 0) {
            this.saveResource(unionEnterprise.getUnionId(), enterpriseDTO);
        }
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<UnionEnterpriseDTO> getUnionEnterpriseByUnionEnterpriseId(Integer unionEnterrpiseId) {
        TabSysUnionEnterprise enterprise = this.unionEnterpriseService.getUnionEnterpriseByUnionEnterpriseId(unionEnterrpiseId);
        UnionEnterpriseDTO enterpriseDTO = EntityUtil.changeEntityByJSON(UnionEnterpriseDTO.class, enterprise);
        this.getResource(enterprise.getUnionId(), enterpriseDTO);
        return ServiceResponse.success(enterpriseDTO);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> delUnionEnterprise(Integer unionId, Integer enterpriseId) {
        int i = this.unionEnterpriseService.delUnionEnterprise(unionId);
        if(i > 0){
            int re = this.unionEnterpriseResourceService.delResource(unionId, enterpriseId);
            if(re == 0){
                throw new RuntimeException();
            }
        }
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<Integer> delUnionEnterpriseSource(Long resourceId, Integer enterpriseId) {
        int i = this.unionEnterpriseResourceService.delResource(resourceId);
        if(i > 0){
            Integer unionEnterpriseId = this.unionEnterpriseResourceService.getUnionEnterpriseIdByResouce(resourceId);
            this.storeAuthorizationApiService.cancelStoreAll(enterpriseId, unionEnterpriseId);
        }
        return ServiceResponse.success(i);
    }

    @Override
    public ServiceResponse<UnionEnterpriseDTO> getUnionEnterpriseById(Integer unionId) {
        TabSysUnionEnterprise enterprise = this.unionEnterpriseService.getUnionEnterpriseById(unionId);
        UnionEnterpriseDTO enterpriseDTO = EntityUtil.changeEntityByJSON(UnionEnterpriseDTO.class, enterprise);
        this.getResource(unionId, enterpriseDTO);
        return ServiceResponse.success(enterpriseDTO);
    }

    @Override
    public ServiceResponse<Page<UnionEnterpriseDTO>> pageUnionEnterprise(String search, Integer enterpriseId, Integer pageNum, Integer pageSize, Integer authorizationStatus) {
        PageHelper.startPage(pageNum, pageSize);
        com.github.pagehelper.Page<TabSysUnionEnterprise> page = this.unionEnterpriseService.pageUnionEnterprise(enterpriseId, authorizationStatus, search);
        return ServiceResponse.success(PageHelperUtils.changePageHelperToCurrentPage(page, UnionEnterpriseDTO.class));
    }

    @Override
    public ServiceResponse<Void> updateStoreAuthorizationStatus(int authOrizationStatus, Integer enterpriseId, String message, String key) {
        this.unionEnterpriseResourceService.updateAuthorizationStatus(authOrizationStatus, enterpriseId,
                ResourceTypeEnum.STORE_RESOURCE.getCode(), message , key);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseResourceDTO>> listStoreResourceByEnterpriseId(Integer enterpriseId) {
        List<UnionEnterpriseResourceDTO> longList = this.unionEnterpriseResourceService.listResourceByEnterpriseIdAndType(enterpriseId, ResourceTypeEnum.STORE_RESOURCE.getCode());
        return ServiceResponse.success(longList);
    }

    @Override
    public ServiceResponse<List<UnionEnterpriseResourceDTO>> listResourceByUnionId(Integer unionId) {
        List<TabSysUnionEnterpriseResource> list = this.unionEnterpriseResourceService.listResourceByUnionId(unionId);
        return ServiceResponse.success(EntityUtil.changeEntityListByJSON(UnionEnterpriseResourceDTO.class, list));
    }

    @Override
    public ServiceResponse<Void> reAuthorizationResource(Long resourceId, Integer resourceType) {
        if(resourceType.equals(ResourceTypeEnum.STORE_RESOURCE.getCode())){
            String key = this.unionEnterpriseResourceService.getKeyByResourceId(resourceId);
            this.storeAuthorizationApiService.reAuthStore(key);
        }else if(resourceType.equals(ResourceTypeEnum.GOODS_RESOURCE.getCode())){

        }else if(resourceType.equals(ResourceTypeEnum.MEMBER_RESOURCE.getCode())){

        }else if(resourceType.equals(ResourceTypeEnum.ORDER_RESOURCE.getCode())){

        }
        return ServiceResponse.success();
    }

    private int saveResource(Integer unionId, UnionEnterpriseDTO enterpriseDTO){
        int i = 0;
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getMemberResourceId(),
                ResourceTypeEnum.MEMBER_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getStoreResourceId(),
                ResourceTypeEnum.STORE_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }else {
            this.storeAuthorizationApiService.authStore(enterpriseDTO.getStoreResourceId().intValue(),
                    enterpriseDTO.getUnionEnterpriseId(), enterpriseDTO.getEnterpriseId());
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getGoodsResourceId(),
                ResourceTypeEnum.GOODS_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        i = this.unionEnterpriseResourceService.saveResource(unionId, enterpriseDTO.getOrderResourceId(),
                ResourceTypeEnum.ORDER_RESOURCE.getCode(), enterpriseDTO.getEnterpriseId());
        if( i == 0){
            throw new RuntimeException();
        }
        return i;
    }

    private void getResource(Integer unionId, UnionEnterpriseDTO enterpriseDTO){
        TabSysUnionEnterpriseResource resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.MEMBER_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setMemberResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.STORE_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setStoreResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.GOODS_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setGoodsResourceId(resource.getResource());
        }
        resource = this.unionEnterpriseResourceService.getByUnionIdAndType(unionId, ResourceTypeEnum.ORDER_RESOURCE.getCode());
        if(resource != null){
            enterpriseDTO.setOrderResourceId(resource.getResource());
        }
    }
}
