package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.dto.ResourceGroupDTO;
import com.gic.auth.dto.UserResourceDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.service.ResourceGroupApiService;
import com.gic.auth.service.UserResourceApiService;
import com.gic.auth.service.UserResourceService;
import com.gic.auth.service.UserService;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
/**
 * 
 * @ClassName: UserResourceApiServiceImpl
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 9:51
 */
@Service("userResourceApiService")
public class UserResourceApiServiceImpl implements UserResourceApiService{
    @Autowired
    private UserResourceService userResourceService;
    @Autowired
    private UserService userService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;
    @Override
    public ServiceResponse<Void> save(UserResourceDTO dto) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(dto, UserResourceDTO.SaveUserResourceValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        TabSysUser user = userService.getUserById(dto.getUserId());
        if (user == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "用户数据有误，查无数据");
        }

        dto.setCreateTime(new Date());
        dto.setUpdateTime(new Date());
        dto.setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS);

        userResourceService.save(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<UserResourceDTO> getResourceByUserId(Integer userId) {
        List<UserResourceDTO> list = this.userResourceService.listUserResourceByUserId(userId);
        if(CollectionUtils.isNotEmpty(list)){
            return ServiceResponse.success(list.get(0));
        }
        return ServiceResponse.success(new UserResourceDTO());
    }

    @Override
    public ServiceResponse<ResourceGroupDTO> getResourceGroupByUserId(Integer userId) {
        ServiceResponse<UserResourceDTO> response = getResourceByUserId(userId);
        if (response.isSuccess()) {
            UserResourceDTO userResourceDTO = response.getResult();
            if (userResourceDTO != null) {
                Integer resourceId = userResourceDTO.getResourceId();
                ServiceResponse<ResourceGroupDTO> resourceRes = resourceGroupApiService.getResourceGroup(resourceId);
                if (resourceRes.isSuccess()) {
                    ResourceGroupDTO result = resourceRes.getResult();
                    if (result != null) {
                        return ServiceResponse.success(result);
                    }
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有资源组数据");
                }
                return ServiceResponse.failure(resourceRes.getCode(), resourceRes.getMessage());
            }
        }
        return ServiceResponse.failure(response.getCode(), response.getMessage());
    }

}
