package com.gic.auth.utils.menu;

import com.gic.auth.constant.DeleteFlagConstants;
import com.gic.auth.constant.MenuProjectConstants;
import com.gic.auth.constant.SignConstants;
import com.gic.auth.dto.MenuAuthDepartDTO;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.MenuAuthDepartService;
import com.gic.auth.service.MenuService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 权限菜单操作
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/16 10:31 AM 
 */
@Component
public abstract class AbstractMenuOperation {
    @Autowired
    protected MenuApiService menuApiService;
    @Autowired
    protected MenuAuthDepartService menuAuthDepartService;
    @Autowired
    protected MenuService menuService;

    /**
     * 权限操作
     * @param menuDTO
     * @return 如果有数据，表示错误信息
     */
    public abstract String operate(MenuDTO menuDTO);

    /**
     * gic基础版本/应用版本 数据处理
     * @param menuDTO
     */
    protected void setMenuVersion(MenuDTO menuDTO) {
        if (MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            String[] menuVersionArr = menuDTO.getMenuVersion().split(SignConstants.UNDERLINE);
            StringBuilder result = new StringBuilder();

            Map<String, String> gicDefaultVersion = menuApiService.getGicDefaultVersion().getResult();
            for (String str : menuVersionArr) {
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                result.append(gicDefaultVersion.get(str)).append(SignConstants.SLASH);
            }
            String menuVersionStr = result.toString();
            menuDTO.setMenuVersionName(menuVersionStr.substring(0, menuVersionStr.length() - 1));
        } else {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            menuDTO.setMenuVersionName(
                    menuDTO.getMenuVersionName().replaceAll(SignConstants.COMMA, SignConstants.SLASH));
        }
    }

    /**
     * 设置 权限适用 数据 管理员/超管/达摩人员
     * @param menuDTO
     */
    protected void setMenuAuthType(MenuDTO menuDTO) {
        Integer authType = menuDTO.getAuthType();
        if (authType == null) {
            authType = 1;
        }
        menuDTO.setAuthType(authType);
        if (authType != 1) {
            List<MenuAuthDepartDTO> list = menuDTO.getMenuAuthDepartList();
            if (CollectionUtils.isNotEmpty(list)) {
                Date now = new Date();
                menuDTO.setMenuAuthDepartList(
                        list.stream().map(e -> e.setMenuCode(menuDTO.getMenuCode())
                                .setDeleteFlag(DeleteFlagConstants.NORMAL_STATUS)
                                .setCreateTime(now)
                                .setUpdateTime(now)).collect(Collectors.toList()));
            }
        }
    }
}
