package com.gic.auth.utils.menu;

import org.springframework.stereotype.Component;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.constant.MenuProjectConstants;
import com.gic.auth.constant.MenuTypeEnum;
import com.gic.auth.dto.MenuDTO;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import org.springframework.transaction.annotation.Transactional;

/**
 * gic菜单新增
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/16 10:36 AM 
 */
@Component
public class GicMenuSave extends AbstractMenuOperation {

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String operate(MenuDTO menuDTO) {
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO,  MenuDTO.SavePageValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid.getMessage();
        }
        //验证code是否重复
        if (menuService.isRepeatMenuCode(menuDTO.getMenuCode(), null)) {
            return "code:" + menuDTO.getMenuCode() + "重复";
        }
        //组装数据
        setMenuVersion(menuDTO);
        String project = MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject()) ? menuDTO.getProject() : null;
        int maxSort = menuService.getMaxSortByParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, project);
        menuDTO.setSort(maxSort + 1);
        menuDTO.setLevel(1);
        menuDTO.setParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID);
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        setMenuAuthType(menuDTO);
        menuService.savePage(menuDTO);
        menuAuthDepartService.saveFetch(menuDTO.getMenuAuthDepartList(), menuDTO.getMenuCode());
        return null;
    }
}
