package com.gic.auth.utils.resourcegroup;

import com.alibaba.fastjson.JSON;
import com.gic.auth.constant.ResourceHoldEnum;
import com.gic.auth.dto.CustomUserAreaDTO;
import com.gic.auth.dto.ResourceGroupCustomUserDTO;
import com.gic.auth.entity.TabSysResourceGroup;
import org.apache.commons.lang.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户域配置
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/12 2:16 PM 
 */
public class CustomResourceData implements IResourceGroupData{

    @Override
    public <T> T getResourceData(TabSysResourceGroup resourceGroup) {
        //组装用户域配置信息
        List<ResourceGroupCustomUserDTO> customUserList = JSON.parseArray(resourceGroup.getCustomUserData(), ResourceGroupCustomUserDTO.class);
        CustomUserAreaDTO result = new CustomUserAreaDTO()
                .setType(ResourceHoldEnum.DART.getCode())
                .setCustomUser(resourceGroup.getCustomUser())
                .setWeimobOrderArea(resourceGroup.getWeimobOrderArea())
                .setStoreOrderArea(resourceGroup.getStoreOrderArea())
                .setCustomUserList(customUserList);
        if (StringUtils.isNotBlank(resourceGroup.getSmsId())) {
            result.setSmsId(Arrays.stream(resourceGroup.getSmsId()
                    .split(","))
                    .mapToLong(e -> Long.valueOf(e))
                    .boxed()
                    .collect(Collectors.toList()));
        }
        if (StringUtils.isNotBlank(resourceGroup.getWeimobStoreId())) {
            result.setWeimobStoreId(Arrays.stream(resourceGroup.getWeimobStoreId().split(","))
                    .mapToLong(e -> Long.valueOf(e))
                    .boxed()
                    .collect(Collectors.toList()));
        }
        return (T) result;
    }

    @Override
    public <T> T getResourceDataForAdmin() {
        return (T) new CustomUserAreaDTO().setType(ResourceHoldEnum.ALL.getCode());
    }

    @Override
    public <T> T getResourceDataWhenNoExist() {
        return (T) new CustomUserAreaDTO().setType(ResourceHoldEnum.DART.getCode()).setCustomUser(0);
    }
}
