package com.gic.auth.web.controller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.gic.auth.dto.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.LoginUserTypeEnum;
import com.gic.auth.service.*;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.Md5Util;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.base.*;
import com.gic.enterprise.constant.EnterpriseStatusTypeEnum;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.context.UserContext;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.member.config.api.service.AppletsConfigApiService;
import com.gic.member.config.api.service.MemberCardApiService;
import com.gic.member.config.api.service.ServiceConfigApiService;
import com.gic.redis.data.util.RedisUtil;

/**
 *
 * @ClassName: LoginController
 * @Description:
 * @author zhiwj
 * @date 2020-09-10 10:01
 */
@RestController
public class LoginController {
    private final static Logger logger = LogManager.getLogger(LoginController.class);

    private static Integer MD5_LENGTH = 32;

    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private OperationUserApiService operationUserApiService;
    @Autowired
    private UserResourceApiService userResourceApiService;
    @Autowired
    private MemberCardApiService memberCardApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;
    @Autowired
    private ServiceConfigApiService serviceConfigApiService;
    @Autowired
    private ResourceGroupApiService resourceGroupApiService;

    private static final String MENU_LOCAL_CACHE_KEY = "auth:menu:list:all";

    /**
     * 用户第一次登陆需要关注达摩服务号，为了获取用户的openId
     * @param nationCode
     * @param phoneNumber
     * @param enterpriseId
     * @param password
     * @return
     */
    @RequestMapping("get-user-qrcode")
    @HeaderSignIgnore
    @IgnoreLogin
    public RestResponse getUserQrcode(String nationCode, String phoneNumber, Integer enterpriseId, String password) {
        if (StringUtils.isBlank(password) || StringUtils.isBlank(phoneNumber) || enterpriseId == null) {
            return EnterpriseRestResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (StringUtils.isBlank(nationCode)) {
            nationCode = Constants.NATION_CODE;
        }
        Md5Util md5 = new Md5Util();
        // password 自身作为盐值
        if (password.length() != LoginController.MD5_LENGTH) {
            password = md5.encrypt(password + password);
        }
        ServiceResponse<UserDTO> login = this.userApiService.login(nationCode, phoneNumber, enterpriseId, password);
        if (login.isSuccess()) {
            UserDTO userDTO = login.getResult();
            if (userDTO == null) {
                return EnterpriseRestResponse.failure(ErrorCode.LOGIN_ERR);
            }
            if (StringUtils.isNotBlank(userDTO.getOpenId())) {
                //如果已经关注过，不再生成
                return RestResponse.success();
            }
            return ResultControllerUtils.commonResult(userApiService.getUserQrcode(userDTO.getUserId(), enterpriseId));
        }
        return RestResponse.success();
    }

    @RequestMapping("list-enterprise-by-phone")
    public RestResponse listUserEnterprise(String phoneNumber, String nationCode) {
        if (StringUtils.isBlank(phoneNumber)) {
            return EnterpriseRestResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (StringUtils.isBlank(nationCode)) {
            nationCode = Constants.NATION_CODE;
        }
        ServiceResponse<List<UserDTO>> listServiceResponse = this.userApiService.listUserByPhoneNumber(phoneNumber,
                nationCode);
        if (listServiceResponse.isSuccess()) {
            List<UserDTO> result = listServiceResponse.getResult();
            if (CollectionUtils.isEmpty(result)) {
                return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
            } else {
                List<Integer> idList = new ArrayList<>();
                for (UserDTO userDTO : result) {
                    //gic管理员，不查询运营实施
                    if (userDTO.getLoginType().intValue() == LoginUserTypeEnum.GIC_USER.getCode()) {
                        idList.add(userDTO.getEnterpriseId());
                    }
                }
                ServiceResponse<List<EnterpriseDTO>> serviceResponse = this.enterpriseApiService
                        .listEnterpriseByIds(idList);
                if (serviceResponse.isSuccess()) {
                    List<EnterpriseDTO> list = serviceResponse.getResult();
                    if (CollectionUtils.isEmpty(list)) {
                        return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
                    } else {
                        return RestResponse.success(list);
                    }
                }
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("login")
    public RestResponse login(String nationCode, String phoneNumber, Integer enterpriseId, String password) {
        if (StringUtils.isBlank(password) || StringUtils.isBlank(phoneNumber) || enterpriseId == null) {
            return EnterpriseRestResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (StringUtils.isBlank(nationCode)) {
            nationCode = Constants.NATION_CODE;
        }
        long startTime = System.currentTimeMillis();
        Md5Util md5 = new Md5Util();
        // password 自身作为盐值
        if (password.length() != LoginController.MD5_LENGTH) {
            password = md5.encrypt(password + password);
        }
        ServiceResponse<UserDTO> login = this.userApiService.login(nationCode, phoneNumber, enterpriseId, password);
        if (login.isSuccess()) {
            UserDTO userDTO = login.getResult();
            if (userDTO == null) {
                return EnterpriseRestResponse.failure(ErrorCode.LOGIN_ERR);
            }

            //判断，如果是临时账号，还需要判断是否过期
            boolean isExpireAccount = userDTO.getAccountType() == 2 && (userDTO.getAccountDueDate().getTime() < System.currentTimeMillis());
            if (isExpireAccount) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "账号已失效，请联系管理员！");
            }

            UserDetail userDetail = new UserDetail();
            userDetail.setUserId(userDTO.getUserId());
            userDetail.setUserInfo(EntityUtil.changeEntityNew(UserInfo.class, userDTO));
            ServiceResponse<EnterpriseDTO> enterprise = this.enterpriseApiService.getEnterpriseById(enterpriseId);
            if (enterprise.isSuccess()) {
                EnterpriseDTO enterpriseDTO = enterprise.getResult();
                Integer serviceStatus = enterpriseDTO.getServiceStatus();
                //如果停用
                if (serviceStatus != null && serviceStatus.intValue() == EnterpriseStatusTypeEnum.DISABLE.getCode()) {
                    return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "您的GIC商户后台已被停用，请及时联系达摩项目经理！");
                }
                if (enterpriseDTO != null) {
                    userDetail.setEnterpriseInfo(EntityUtil.changeEntityNew(EnterpriseInfo.class, enterpriseDTO));
                    //登录用户所拥有的权限菜单和操作项（接口）
                    ServiceResponse<List<MenuDTO>> menuResult = menuApiService.listUserMenu(userDetail.getUserId(),
                            enterpriseDTO.getVersionCode());
                    if (menuResult.isSuccess()) {
                        List<MenuDTO> menuList = menuResult.getResult();
                        Set<Integer> menuIdSet = new HashSet<>(2);
                        if (CollectionUtils.isNotEmpty(menuList)) {
                            menuIdSet = menuList.stream().filter(e -> StringUtils.isNotBlank(e.getMenuUrl()))
                                    .map(e -> e.getMenuId()).collect(Collectors.toSet());
                        }
                        //塞用户权限菜单值
                        userDetail.setMenuIdSet(menuIdSet);
                        //塞本地缓存
                        setAllMenuToLocaleCache();
                        //设置用户资源组
                        this.setUserResource(userDetail, userDetail.getUserId());
                    }
                    //塞缓存
                    UserDetailUtils.setUserDetail(userDetail);
                    logger.info("登录所花时间:{}", System.currentTimeMillis() - startTime);
                    return RestResponse.success(userDetail);
                }
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("get-login-user-info")
    public RestResponse getLoginUserInfo() {
        return RestResponse.success(UserDetailUtils.getUserDetail().getUserInfo());
    }

    @RequestMapping("logout")
    public RestResponse logout() {
        UserDetailUtils.destory();
        return RestResponse.success();
    }

    /**
     * 运营平台单点登录
     * @Title: login 
     * @Description:
      * @author guojuxing
     * @param request
     * @param response 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @IgnoreLogin
    @RequestMapping("login-for-operation")
    public RestResponse login(HttpServletRequest request, HttpServletResponse response) throws IOException {
        String redirectUrl = request.getParameter("redirectUrl");
        String token = request.getParameter("token");
        response.setStatus(302);
        if (UserContext.getContext().getByToken(token).getUserInfo() != null) {
            //已有登录信息
            response.setHeader("location", redirectUrl);
            //塞缓存
            UserDetailUtils.setUserDetail(UserContext.getContext().getByToken(token));
            return RestResponse.success();
        }
        return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "token错误");
    }

    /**
     * gic平台的菜单数据塞本地缓存
     */
    private void setAllMenuToLocaleCache() {
        ServiceResponse<List<MenuDTO>> response = menuApiService.getAllMenu();
        if (response.isSuccess()) {
            List<MenuDTO> list = response.getResult();
            //gic平台的菜单数据塞本地缓存
            RedisUtil.delLocalCache(MENU_LOCAL_CACHE_KEY);
            RedisUtil.setLocalCache(MENU_LOCAL_CACHE_KEY, EntityUtil.changeEntityListNew(MenuInfo.class, list), null);
        }
    }

    private void setUserResource(UserDetail detail, Integer userId) {
        ServiceResponse<UserResourceDTO> resource = this.userResourceApiService.getResourceByUserId(userId);
        if (resource.isSuccess() && resource.getResult() != null) {
            //查询管理员的资源组信息
            ServiceResponse<ResourceGroupDataDTO> response = resourceGroupApiService.getResourceGroupByUserId(userId);
            if (resource.isSuccess()) {
                ResourceGroupDataDTO resourceGroupDataDTO = response.getResult();

                detail.setCustomUserInfo(EntityUtil.changeEntityByJSON(CustomUserInfo.class, resourceGroupDataDTO.getCustomUserAreaDTO()));
                detail.setStoreResourceInfo(EntityUtil.changeEntityNew(StoreResourceInfo.class, resourceGroupDataDTO.getStoreResourceDTO()));
                detail.setGoodsResourceInfo(EntityUtil.changeEntityNew(GoodsResourceInfo.class, resourceGroupDataDTO.getGoodsResourceDTO()));
            } else {
                detail.setCustomUserInfo(new CustomUserInfo());
                detail.setGoodsResourceInfo(new GoodsResourceInfo());
                detail.setStoreResourceInfo(new StoreResourceInfo());
            }
        }
    }
}
