package com.gic.auth.web.controller;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.gic.auth.web.utils.EnterpriseBasicAppUtils;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.MenuLevelConstants;
import com.gic.auth.constant.MenuProjectConstants;
import com.gic.auth.constant.SignConstants;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.dto.RoleDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.RoleListQO;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.RoleApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.vo.RoleDetailVO;
import com.gic.auth.web.vo.RoleSelectVO;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
/**
 *
 * @ClassName: RoleController
 * @Description:
 * @author zhiwj
 * @date 2020-09-10 10:01
 */
@RestController
@RequestMapping("/role")
public class RoleController {

    @Autowired
    private RoleApiService roleApiService;
    @Autowired
    private MenuApiService menuApiService;
    @Autowired
    private UserApiService userApiService;

    @RequestMapping("/get-detail")
    public RestResponse getDetail(Integer id) {
        ServiceResponse<RoleDTO> roleResult = roleApiService.getById(id);
        if (roleResult.isSuccess()) {
            RoleDTO roleDTO = roleResult.getResult();
            RoleDetailVO vo = EntityUtil.changeEntityNew(RoleDetailVO.class, roleDTO);

            String menuIds = roleDTO.getMenuIds();
            String[] menuIdArr = menuIds.split(SignConstants.COMMA);
            List<Integer> menuIdList = new ArrayList<>(menuIdArr.length);
            for (String menuId : menuIdArr) {
                menuIdList.add(Integer.parseInt(menuId));
            }
            ServiceResponse<List<MenuDTO>> menuListResult = menuApiService.listByMenuIdList(menuIdList);
            Map<String, Object> gicMap = new HashMap<>(16);
            if (menuListResult.isSuccess()) {
                List<MenuDTO> menuDTOList = menuListResult.getResult();
                for (MenuDTO menuDTO : menuDTOList) {
                    gicMap.put(menuDTO.getMenuId().toString(), 1);
                }
            }

            ServiceResponse<List<MenuDTO>> gicResult = menuApiService.listMenuTreeForRoleOfGic(
                    UserDetailUtils.getUserDetail().getEnterpriseInfo().getVersionCode());
            vo.setGicMap(
                    changeListToSelectTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, gicResult.getResult(), gicMap));

            //app已购买应用
            ServiceResponse<List<MenuDTO>> appResult = menuApiService
                    .listAppPageOfHasBuyForRole(UserDetailUtils.getUserDetail().getEnterpriseId());
            //过滤商户基础应用
            List<MenuDTO> appMenuList = appResult.getResult();
            if (CollectionUtils.isNotEmpty(appMenuList)) {
                appMenuList = appMenuList.stream()
                        .filter(e -> !EnterpriseBasicAppUtils.ENTERPRISE_BASIC_APP_CODE.equals(e.getMenuCode()))
                        .collect(Collectors.toList());
            }

            vo.setAppMap(
                    changeListToSelectTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, appMenuList, gicMap));
            return RestResponse.success(vo);
        }
        return EnterpriseRestResponse.failure(roleResult);
    }

    @RequestMapping("/save")
    public RestResponse save(RoleDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(roleApiService.save(dto), OperationResultUtils.LOG_NEW + "角色",
                dto.getRoleName());
    }

    @RequestMapping("/update")
    public RestResponse update(RoleDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return OperationResultUtils.operationResult(roleApiService.update(dto), OperationResultUtils.LOG_EDIT + "角色",
                dto.getRoleName());
    }

    @RequestMapping("/delete")
    public RestResponse delete(Integer id) {
        ServiceResponse<String> result = roleApiService.delete(id);
        String operationObject = result.isSuccess() ? result.getResult() : null;
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "角色", operationObject);
    }

    @RequestMapping("/list-role")
    public RestResponse listRole(RoleListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(roleApiService.pageRole(params));
    }

    @RequestMapping("/list-role-no-page")
    public RestResponse listRoleNoPage() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(roleApiService.listRole(enterpriseId), RoleSelectVO.class);
    }

    @RequestMapping("/list-menu")
    public RestResponse listMenu() {
        ServiceResponse<List<MenuDTO>> menuResponse = menuApiService.getSuperAdminGicMenu(
                UserDetailUtils.getUserDetail().getEnterpriseId());
        if (menuResponse.isSuccess()) {
            return RestResponse.success(menuResponse.getResult());
        } else {
            return EnterpriseRestResponse.failure(menuResponse);
        }
    }

    /**
     * 查询授权子管理菜单权限列表数据
     * @Title: updateAppPage 
     * @Description:
      * @author guojuxing
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/list-gic-menu-tree")
    public RestResponse listMenuTree() {
        ServiceResponse<List<MenuDTO>> result = menuApiService.listMenuTreeForRoleOfGic(
                UserDetailUtils.getUserDetail().getEnterpriseInfo().getVersionCode());
        if (result.isSuccess()) {
            return RestResponse.success(changeListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, result.getResult()));
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 查询授权子管理员应用菜单权限数据
     * @return
     */
    @RequestMapping("/list-app-menu-tree")
    public RestResponse listAppMenuTree() {
        ServiceResponse<List<MenuDTO>> result = menuApiService
                .listAppPageOfHasBuyForRole(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (result.isSuccess()) {
            List<MenuDTO> menuList = result.getResult();
            if (CollectionUtils.isNotEmpty(menuList)) {
                menuList = menuList.stream()
                        .filter(e -> !EnterpriseBasicAppUtils.ENTERPRISE_BASIC_APP_CODE.equals(e.getMenuCode()))
                        .collect(Collectors.toList());
            }
            return RestResponse.success(changeListToTree(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, menuList));
        }
        return EnterpriseRestResponse.failure(result);
    }

    public static List<Map<String, Object>> changeListToTree(int storeGroupId, List<MenuDTO> list) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (MenuDTO menuDTO : list) {
                if (storeGroupId == menuDTO.getParentId().intValue()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("project", menuDTO.getProject());
                    data.put("menuType", menuDTO.getMenuType());
                    List<Map<String, Object>> children = changeListToTree(menuDTO.getMenuId(), list);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }

    public static List<Map<String, Object>> changeListToSelectTree(int storeGroupId, List<MenuDTO> list,
            Map<String, Object> menuIdMap) {
        List<Map<String, Object>> result = new ArrayList<>();
        if (list != null) {
            for (MenuDTO menuDTO : list) {
                if (storeGroupId == menuDTO.getParentId().intValue()) {
                    Map<String, Object> data = new HashMap<>(16);
                    data.put("menuId", menuDTO.getMenuId());
                    data.put("menuName", menuDTO.getMenuName());
                    data.put("project", menuDTO.getProject());
                    data.put("menuType", menuDTO.getMenuType());
                    if (menuIdMap.containsKey(menuDTO.getMenuId().toString())) {
                        //如果存在，则说明钩上了
                        data.put("isSelect", true);
                    } else {
                        data.put("isSelect", false);
                    }
                    List<Map<String, Object>> children = changeListToSelectTree(menuDTO.getMenuId(), list, menuIdMap);
                    if (children == null || children.size() < 1) {
                        data.put("children", null);
                    } else {
                        data.put("children", children);
                    }
                    result.add(data);
                }
            }
        }
        return result;
    }
}
