package com.gic.auth.web.controller;

import com.gic.auth.dto.wechat.WechatUserDTO;
import com.gic.auth.web.vo.SuperUserVO;
import com.gic.auth.web.vo.UserOfGicVO;
import com.gic.commons.annotation.HeaderSignIgnore;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.base.EnterpriseInfo;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.service.EnterpriseApiService;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.UserConstants;
import com.gic.auth.dto.AuthCodeDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.utils.UserPasswordUtils;
import com.gic.auth.web.vo.LoginUserVO;
import com.gic.auth.web.vo.UserDetailVO;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.Md5Util;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.OperationResultUtils;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.marketing.process.api.service.sms.SmsSendApiService;

import java.util.Calendar;
import java.util.Date;
/**
 * 
 * @ClassName: UserController
 * @Description:  
 * @author zhiwj
 * @date 2020-09-10 10:01
 */
@RestController
@RequestMapping("/user")
public class UserController {

    private final Logger logger = LogManager.getLogger(UserController.class);
    private static Integer MD5_LENGTH = 32;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private SmsSendApiService smsSendApiService;
    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private EnterpriseApiService enterpriseApiService;

    @RequestMapping("/save-or-update-user")
    public RestResponse editUser(@Validated({ UserDTO.UserQoValid.class }) UserDTO userDTO) {
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin().intValue() == 0) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "子管理员不能编辑");
        }
        Integer operationPassword = userDTO.getOperPasswordType();
        userDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());

        boolean isNeedSendSms = false;
        //发送短信的参数
        String[] smsArr = new String[0];
        if (operationPassword != null && operationPassword.intValue() == 1) {
            //如果是1，则不用验证密码

        } else {
            String password = UserPasswordUtils.validPassword(userDTO.getPasswordType(), userDTO.getPassword());
            userDTO.setPassword(UserPasswordUtils.getEncryptPassword(password));
            isNeedSendSms = UserConstants.CREATE_AUTO == userDTO.getPasswordType().intValue();
            if (isNeedSendSms) {
                //发送短信，通知自动生成密码
                if (StringUtils.isBlank(userDTO.getPhoneAreaCode())) {
                    userDTO.setPhoneAreaCode(Constants.NATION_CODE);
                }
                smsArr = new String[] { UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseName(), password };
            }
        }
        ServiceResponse userResult;
        //普通用户
        userDTO.setSuperAdmin(0);

        //sms场景ID
        String smsSenceId = "GICSHHT003";
        if (userDTO.getUserId() == null) {
            userResult = userApiService.saveUser(userDTO);
        } else {
            smsSenceId = "GICSHHT004";
            userResult = userApiService.editUser(userDTO);
        }
        if (userResult.isSuccess()) {
            if (isNeedSendSms) {
                ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms(smsSenceId,
                        UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId(),
                        userDTO.getPhoneAreaCode(), userDTO.getPhoneNumber(), smsArr);
                if (!smsSendResult.isSuccess()) {
                    logger.warn(smsSendResult.getMessage());
                }
            }
            if (userDTO.getUserId() == null) {
                LogUtils.createLog(OperationResultUtils.LOG_NEW + "管理员", userDTO.getUserName());
            } else {
                LogUtils.createLog(OperationResultUtils.LOG_EDIT + "管理员", userDTO.getUserName());
            }
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(userResult);
        }
    }

    /**
     * 用户详情
     *
     * @param userId
     * @return com.gic.commons.webapi.reponse.RestResponse
     *
     * @Title: getUserById
     * @Description:   * @author guojuxing
     */
    @RequestMapping("/get-user-by-id")
    public RestResponse getUserById(Integer userId) {
        return ResultControllerUtils.commonResultOne(userApiService.getUserById(userId), UserDetailVO.class);
    }

    @RequestMapping("/delete-user-by-id")
    public RestResponse delete(Integer userId) {
        ServiceResponse<String> result = userApiService.delete(userId);
        return OperationResultUtils.operationResult(result, OperationResultUtils.LOG_DELETE + "管理员",
                OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-user")
    public RestResponse listUser(UserListQO params) {
        if (params.getAccountType() == null) {
            //默认查询永久账号
            params.setAccountType(1);
        }
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.pageUser(params));
    }

    /**
     * 个人资料
     * @Title: getLoginUser 
     * @Description:
      * @author guojuxing  
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/get-login-user")
    public RestResponse getLoginUser() {
        LoginUserVO vo;
        ServiceResponse<UserDTO> userResult = userApiService.getUserInfoById(UserDetailUtils.getUserDetail().getUserId());
        if (userResult.isSuccess()) {
            UserDTO userDTO = userResult.getResult();
            vo = EntityUtil.changeEntityNew(LoginUserVO.class, userDTO);

            //查询微信信息
            ServiceResponse<WechatUserDTO> memberInfoRes = userApiService.getMemberInfoByOpenId(userDTO.getOpenId());
            if (memberInfoRes.isSuccess()) {
                WechatUserDTO wechat = memberInfoRes.getResult();
                vo.setNickname(wechat.getNickname());
                vo.setHeadimgurl(wechat.getHeadimgurl());
            }
        } else {
            vo = new LoginUserVO();
        }
        //商户logo
        EnterpriseInfo enterpriseInfo = UserDetailUtils.getUserDetail().getEnterpriseInfo();
        vo.setLogo(enterpriseInfo.getLogo());
        vo.setEnterpriseName(enterpriseInfo.getEnterpriseName());
        vo.setCompanyName(enterpriseInfo.getCompanyName());
        return RestResponse.success(vo);
    }

    /**
     * 换绑
     * @return
     */
    @RequestMapping("/change-bind")
    public RestResponse changeBind(Integer userId) {
        return ResultControllerUtils.commonResult(userApiService.getUserQrcode(userId == null ? UserDetailUtils.getUserDetail().getUserId() : userId,
                UserDetailUtils.getUserDetail().getEnterpriseId()));
    }

    @RequestMapping("send-auth-code-to-modify-phone")
    public RestResponse sendAuthCode(String phone, String nationCode) {
        if (StringUtils.isBlank(nationCode)) {
            nationCode = Constants.NATION_CODE;
        }
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        //生成4位数字验证码
        String authCode = CreateRandomUtils.getStringRandom(4);
        authCodeDTO.setAuthCode(authCode);
        authCodeDTO.setExpirationTime(setAuthCodeExpireTime(5));
        ServiceResponse<Integer> result = authCodeApiService.saveAuth(authCodeDTO);
        if (result.isSuccess()) {
            Integer authCodeId = result.getResult();
            //发送手机验证码
            String[] smsArr = new String[] { authCode };
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICSHHT005",
                    UserDetailUtils.getUserDetail().getEnterpriseInfo().getEnterpriseId(), nationCode, phone, smsArr);
            if (!smsSendResult.isSuccess()) {
                return EnterpriseRestResponse.failure(smsSendResult);
            } else {
                return RestResponse.success(authCodeId);
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    @RequestMapping("modify-phone")
    public RestResponse modifyPhone(String phone, String nationCode, String authCode, Integer authCodeId) {
        ServiceResponse authCodeResult = authCodeApiService.validateAuthCode(authCodeId, authCode);
        if (authCodeResult.isSuccess()) {
            UserDetail userDetail = UserDetailUtils.getUserDetail();
            // password 自身作为盐值
            String password = userDetail.getUserInfo().getPassword();
            ServiceResponse<UserDTO> login = this.userApiService.login(nationCode,
                    userDetail.getUserInfo().getPhoneNumber(), userDetail.getEnterpriseId(), password);
            ServiceResponse<String> result = userApiService.editPhoneOrPassword(login.getResult().getUserId(),
                    nationCode, phone, null);
            if (result.isSuccess()) {
                //过期验证码
                authCodeApiService.expireAuthCode(authCodeId);
            }
            return OperationResultUtils.operationResult(result, "修改手机号",
                    OperationResultUtils.getOperationObject(result));
        }
        return EnterpriseRestResponse.failure(authCodeResult);
    }

    @RequestMapping("send-auth-code-to-modify-password")
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse sendAuthCode(String nationCode, String phoneNumber, Integer enterpriseId) {
        ServiceResponse<UserDTO> response = userApiService.getGicUserByPhoneNumber(nationCode, phoneNumber, enterpriseId);
        if (!response.isSuccess()) {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }
        ServiceResponse<EnterpriseDTO> enterpriseResponse = enterpriseApiService.getEnterpriseById(enterpriseId);

        //生成数字验证码
        AuthCodeDTO authCodeDTO = new AuthCodeDTO();
        authCodeDTO.setEnterpriseId(enterpriseId);
        String authCode = CreateRandomUtils.getStringRandom(4);
        authCodeDTO.setAuthCode(authCode);
        authCodeDTO.setExpirationTime(setAuthCodeExpireTime(1));
        ServiceResponse<Integer> result = authCodeApiService.saveAuth(authCodeDTO);
        if (result.isSuccess()) {
            Integer authCodeId = result.getResult();
            //发送手机验证码
            String[] smsArr = new String[] {
                    enterpriseResponse.getResult().getEnterpriseName(), authCode };
            ServiceResponse<Void> smsSendResult = smsSendApiService.sendPlatformSms("GICSHHT004",
                    enterpriseId,
                    nationCode,
                    phoneNumber, smsArr);
            if (!smsSendResult.isSuccess()) {
                return EnterpriseRestResponse.failure(smsSendResult);
            } else {
                return RestResponse.success(authCodeId);
            }
        }
        return EnterpriseRestResponse.failure(result);
    }

    /**
     * 忘记密码，重置密码根据验证码
     * @param authCode 验证码
     * @param authCodeId 验证码ID
     * @param password
     * @param confirmPassword
     * @param nationCode 手机号区号
     * @param phoneNumber 手机号码
     * @param enterpriseId 商户ID
     * @return
     */
    @RequestMapping("reset-password-by-auth-code")
    @IgnoreLogin
    @HeaderSignIgnore
    public RestResponse resetPasswordByAuthCode(String authCode, Integer authCodeId, String password, String confirmPassword,
                                                String nationCode, String phoneNumber, Integer enterpriseId) {
        //获取用户ID
        ServiceResponse<UserDTO> response = userApiService.getGicUserByPhoneNumber(nationCode, phoneNumber, enterpriseId);
        if (!response.isSuccess()) {
            return RestResponse.failure(response.getCode(), response.getMessage());
        }

        ServiceResponse authCodeResult = authCodeApiService.validateAuthCode(authCodeId, authCode);
        if (authCodeResult.isSuccess()) {
            if (StringUtils.isBlank(password)) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "密码不能为空");
            }
            if (!password.equals(confirmPassword)) {
                return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "密码和确认密码不一致");
            }
            //密码加密
            Md5Util md5 = new Md5Util();
            password = md5.encrypt(password + password);
            userApiService.editPhoneOrPassword(response.getResult().getUserId(), null, null,
                    password);
        }
        return RestResponse.failure(authCodeResult.getCode(), authCodeResult.getMessage());
    }

    @RequestMapping("modify-password")
    public RestResponse modifyPassword(String oldPassword, String newPassword) {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        Md5Util md5 = new Md5Util();
        // password 自身作为盐值
        if (oldPassword.length() != UserController.MD5_LENGTH) {
            oldPassword = md5.encrypt(oldPassword + oldPassword);
        }
        ServiceResponse<UserDTO> login = this.userApiService.login(userDetail.getUserInfo().getPhoneAreaCode(),
                userDetail.getUserInfo().getPhoneNumber(), userDetail.getEnterpriseId(), oldPassword);
        if (!login.isSuccess()) {
            return EnterpriseRestResponse.failure(login);
        }
        if (login.getResult() == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "旧密码错误");
        }
        ServiceResponse<String> result = userApiService.editPhoneOrPassword(login.getResult().getUserId(), null, null,
                md5.encrypt(newPassword + newPassword));
        return OperationResultUtils.operationResult(result, "修改密码", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/list-user-nopage")
    public RestResponse listUserNoPage(UserListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.listUser(params));
    }

    @RequestMapping("/list-gic-user")
    public RestResponse listGicUser() {
        return ResultControllerUtils.commonResult(userApiService.listGicUser(UserDetailUtils.getUserDetail().getEnterpriseId()), UserOfGicVO.class);
    }


    @RequestMapping("/edit-user-name")
    public RestResponse editUserName(String name) {
        return OperationResultUtils.operationResult(userApiService
                .updateOperationUser(UserDetailUtils.getUserDetail().getUserId(), name),
                "修改用户名称",
                UserDetailUtils.getUserDetail().getUserInfo().getPhoneNumber());
    }

    @RequestMapping("/edit-user-position")
    public RestResponse editUserPosition(Integer userPosition) {
        return OperationResultUtils.operationResult(userApiService
                        .updateUserPosition(UserDetailUtils.getUserDetail().getUserId(), userPosition),
                "修改职位",
                UserDetailUtils.getUserDetail().getUserInfo().getPhoneNumber());
    }

    /**
     * 逻辑删除用户对应分组数据
     *
     * @param type
     * @param accountGroupId
     * @param userId
     * @return com.gic.commons.webapi.reponse.RestResponse
     *
     * @Title: removeUserAccountGroup
     * @Description:   * @author guojuxing
     */
    @RequestMapping("/remove-user-account-group")
    public RestResponse removeUserAccountGroup(Integer type, Integer accountGroupId, Integer userId) {
        ServiceResponse<String> result = userApiService.removeAccountGroup(accountGroupId, userId, type);
        return OperationResultUtils.operationResult(result, "移除账号分组", OperationResultUtils.getOperationObject(result));
    }

    @RequestMapping("/get-super-user")
    public RestResponse getSuperAdmin() {
        return ResultControllerUtils.commonResultOne(
                userApiService.getUserByEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId()),
                SuperUserVO.class);
    }

    private Date setAuthCodeExpireTime(Integer minute) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.MINUTE, minute);
        return calendar.getTime();
    }

}
