package com.gic.auth.web.utils;

import com.gic.store.constant.Constants;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.*;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zhiwj
 * @date 2019/7/2
 */
public class ExcelUtils {

    private static final short ROW_HEIGHT = 500;
    private static final int COLUMN_HEIGHT = 5000;

    public static XSSFWorkbook getXssfWorkbook(String sheetName, List<List<String>> everyLines, XSSFWorkbook wb) {
        if (CollectionUtils.isEmpty(everyLines)) {
            return null;
        }
        // 第一步，创建一个XSSFWorkbook，对应一个Excel文件
        if (wb == null) {
            wb = new XSSFWorkbook();
        }


        // 第二步，在workbook中添加一个sheet,对应Excel文件中的sheet
        XSSFSheet sheet = wb.createSheet(sheetName);

        sheet.setDefaultRowHeight(ROW_HEIGHT);
        // 第三步，在sheet中添加表头第0行,注意老版本poi对Excel的行数列数有限制
        XSSFRow row = sheet.createRow(0);
        // 第四步，创建单元格，并设置值表头
        XSSFCellStyle style = wb.createCellStyle();
        XSSFFont font = wb.createFont();
        font.setFontName("微软雅黑");
        style.setFont(font);
        //声明列对象
        XSSFCell cell = null;

        //创建标题
        List<String> titles = everyLines.get(0);
        for (int i = 0; i < titles.size(); i++) {
            cell = row.createCell(i);
            cell.setCellValue(titles.get(i));
            cell.setCellStyle(style);
        }

        //创建内容
        for (int i = 1; i < everyLines.size(); i++) {
            row = sheet.createRow(i);
            for (int j = 0; j < everyLines.get(i).size(); j++) {
                sheet.setColumnWidth(j, COLUMN_HEIGHT);
                //将内容按顺序赋给对应的列对象
                XSSFCell cellTemp = row.createCell(j);
                cellTemp.setCellValue(everyLines.get(i).get(j));
                cellTemp.setCellStyle(style);
            }
        }
        return wb;
    }

    private static XSSFCellStyle getTitleStyle(XSSFWorkbook workbook) {
        XSSFCellStyle style = ExcelUtils.getColumnTopStyle(workbook);
        style.setFillForegroundColor(IndexedColors.LIGHT_GREEN.getIndex());
        return style;
    }

    /**
     * 列头单元格样式
     */
    private static XSSFCellStyle getColumnTopStyle(XSSFWorkbook workbook) {

        // 设置字体
        XSSFFont font = workbook.createFont();
        // 设置字体大小
        font.setFontHeightInPoints((short) 11);
        // 字体加粗
        // 设置字体名字
        font.setFontName("Courier New");
        // 设置样式;
        XSSFCellStyle style = workbook.createCellStyle();
        // 在样式用应用设置的字体;
        style.setFont(font);
        // 设置自动换行;
        style.setWrapText(false);
        return style;

    }

    /**
     * 列数据信息单元格样式
     */
    private static XSSFCellStyle getStyle(XSSFWorkbook workbook) {

        // 设置字体
        XSSFFont font = workbook.createFont();
        // 设置字体名字
        font.setFontName("Courier New");
        font.setColor(IndexedColors.RED.index);
        // 设置样式;
        XSSFCellStyle style = workbook.createCellStyle();
        // 在样式用应用设置的字体;
        style.setFont(font);
        // 设置自动换行;
        style.setWrapText(false);

        return style;
    }

    /**
     * @param row      表示数据从第几行开始 例如：第一行为表头，第二行为数据，row的值为2
     * @param fileType 文件的后缀 例如：xls xlsx
     * @return
     */
    public static List<Map<Integer, String>> readExcel(int row, String fileType, Workbook xssfWorkbook, Map<Integer, Boolean> dateTypeMap) {
        if (com.gic.auth.constant.Constants.XLSX.equals(fileType)) {
            return readXlsx(xssfWorkbook, row, dateTypeMap);
        } else {
            return null;
        }
    }

    public static List<Map<Integer, String>> readXlsx(Workbook xssfWorkbook, int row, Map<Integer, Boolean> dateTypeMap) {
        List<Map<Integer, String>> result = new ArrayList<Map<Integer, String>>();
        SimpleDateFormat format = new SimpleDateFormat(Constants.DATE_FORMAT);
//            XSSFWorkbook xssfWorkbook = new XSSFWorkbook(in);
        for (int numSheet = 0; numSheet < 1; numSheet++) {
            Sheet xssfSheet = xssfWorkbook.getSheetAt(numSheet);
            if (xssfSheet == null || xssfSheet.getLastRowNum() == 1) {
                continue;
            }
            for (int rowNum = row - 1; rowNum <= xssfSheet.getLastRowNum(); rowNum++) {
                Row xssfRow = xssfSheet.getRow(rowNum);
                if (xssfRow == null) {
                    continue;
                }
                int minColix = xssfRow.getFirstCellNum();
                int maxColix = xssfRow.getLastCellNum();

                Map<Integer, String> map = new HashMap<Integer, String>();
                for (int colix = minColix; colix < maxColix; colix++) {
                    Cell xssfCell = xssfRow.getCell(colix);

                    if (xssfCell == null) {
                        map.put(colix, "");
                    } else {

                        Boolean isDate = dateTypeMap.get(colix);
                        if (isDate != null && isDate) {
                            if (xssfCell.getCellTypeEnum() == CellType.NUMERIC) {
                                Date date = xssfCell.getDateCellValue();
                                map.put(colix, format.format(date));
                            } else {
                                xssfCell.setCellType(CellType.STRING);
                                map.put(colix, xssfCell.getStringCellValue());
                            }
                        } else {
                            if (xssfCell.getCellTypeEnum() != CellType.STRING) {
                                xssfCell.setCellType(CellType.STRING);
                            }
                            map.put(colix, xssfCell.getStringCellValue());
                        }
                    }
                }
                if (!isEmpty(map, minColix, maxColix)) {
                    result.add(map);
                }
            }
        }

        return result;
    }

    private static boolean isEmpty(Map<Integer, String> data, int startIndex, int endIndex) {
        boolean isEmplty = true;
        for (int i = startIndex + 1; i <= endIndex; i++) {
            String value = data.get(i);
            if (!StringUtils.isBlank(value)) {
                isEmplty = false;
            }
        }
        return isEmplty;
    }
}